# The MIT License (MIT)
#
# Copyright (c) 2018-2019, Moritz E. Beber.
# Copyright (c) 2018-2019, Institute for Molecular Systems Biology, ETH Zurich.
# Copyright (c) 2018-2019, Novo Nordisk Foundation Center for Biosustainability,
#     Technical University of Denmark.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.


"""Provide the declarative base class for ORM models."""


from sqlalchemy import Column, Integer
from sqlalchemy.ext.declarative import declarative_base

from .mixin import TimestampMixin


class CustomBase(TimestampMixin):
    """
    Define a declarative base class with timestamps for ORM models.

    Attributes
    ----------
    id : int
        The auto-incrementing, integer, primary key in the table.

    """

    id = Column(Integer, primary_key=True, autoincrement=True)

    def __repr__(self):
        """Return a string representation of the object."""
        return f"{type(self).__name__}(id={self.id})"


Base = declarative_base(cls=CustomBase)
